@@ -1,8 +1,18 @@
 from __future__ import annotations
-from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes
-from pyvenafi.tpp.api.api_base import ObjectModel, ApiField
+
 from datetime import datetime
-from typing import List, Literal, TypeVar, Generic
+from typing import (
+    Generic,
+    List,
+    Literal,
+    TypeVar,
+)
+
+from pyvenafi.tpp.api.api_base import (
+    ApiField,
+    ObjectModel,
+)
+from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes
 
 T = TypeVar('T')
 
@@ -49,8 +59,10 @@ class CustomFieldAttributes(ObjectModel):
     values: List[str] = ApiField(alias='Values', default_factory=list)
 
 
-class EnvironmentTemplateDetails(InfoValue[str]):
-    template_values: InfoValue[str] = ApiField(alias='TemplateValues')
+class EnvironmentTemplateDetails(ObjectModel, Generic[T]):
+    info: int = ApiField(alias='Info')
+    template_values: InfoValue[T] = ApiField(alias='TemplateValues')
+    value: T = ApiField(alias='Value')
 
 
 class RightsKeyValue(ObjectModel):
@@ -93,7 +105,7 @@ class AppleTemplate(ObjectModel):
     per_user: bool = ApiField(alias='PerUser')
     read_only: bool = ApiField(alias='ReadOnly')
     state: InfoValue[str] = ApiField(alias='State')
-    target_policy_dn: str = ApiField(alias='TargetPolicyDn')
+    target_policy_dn: str = ApiField(alias='TargetPolicyDN')
     type: TemplateType = ApiField(alias='Type')
     visible_to: Items[str] = ApiField(alias='VisibleTo')
 
@@ -109,15 +121,15 @@ class AppleEnvironment(ObjectModel):
     id: int = ApiField(alias='Id')
     ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
     key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     per_user: bool = ApiField(alias='PerUser')
-    template_dn: str = ApiField(alias='TemplateDn')
+    template_dn: str = ApiField(alias='TemplateDN')
     type: EnvironmentType = ApiField(alias='Type')
 
 
 class CertificateTemplate(ObjectModel):
     allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
-    certificate_authority_dn: InfoValue[str] = ApiField(alias='CertificateAuthorityDn')
+    certificate_authority_dn: InfoValue[str] = ApiField(alias='CertificateAuthorityDN')
     certificate_subject: str = ApiField(alias='CertificateSubject')
     certificate_stage: str = ApiField(alias='CertificateStage')
     certificate_status_text: str = ApiField(alias='CertificateStatusText')
@@ -130,44 +142,48 @@ class CertificateTemplate(ObjectModel):
     id: int = ApiField(alias='Id')
     key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
     key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
     organization: InfoValue[str] = ApiField(alias='Organization')
-    organization_unit: InfoValue[str] = ApiField(alias='OrganizationUnit')
+    organizational_unit: InfoValue[str] = ApiField(alias='OrganizationalUnit')
     per_user: bool = ApiField(alias='PerUser')
     read_only: bool = ApiField(alias='ReadOnly')
-    san_email: InfoValue[str] = ApiField(alias='SanEmail')
+    san_email: InfoValue[str] = ApiField(alias='SANEmail')
     state: InfoValue[str] = ApiField(alias='State')
-    target_policy_dn: str = ApiField(alias='TargetPolicyDn')
+    target_policy_dn: str = ApiField(alias='TargetPolicyDN')
     type: TemplateType = ApiField(alias='Type')
     visible_to: Items[str] = ApiField(alias='VisibleTo')
 
 
 class CertificateEnvironment(ObjectModel):
     allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
-    ca_specific_attributes: Items[str] = ApiField(alias='CaSpecificAttributes')
-    certificate_authority_dn: InfoValue[str] = ApiField(alias='CertificateAuthorityDn')
-    certificate_subject: str = ApiField(alias='CertificateSubject')
+    ca_specific_attributes: Items[str] = ApiField(alias='CASpecificAttributes')
+    certificate_authority_dn: EnvironmentTemplateDetails[str] = ApiField(alias='CertificateAuthorityDN')
+    certificate_dn: str = ApiField(alias='CertificateDN')
+    certificate_status: int = ApiField(alias='CertificateStatus')
+    certificate_status_text: int = ApiField(alias='CertificateStatusText')
+    certificate_subject: EnvironmentTemplateDetails[str] = ApiField(alias='CertificateSubject')
     certificate_template: CertificateTemplate = ApiField(alias='CertificateTemplate')
-    city: InfoValue[str] = ApiField(alias='City')
-    country: InfoValue[str] = ApiField(alias='Country')
+    city: EnvironmentTemplateDetails[str] = ApiField(alias='City')
+    country: EnvironmentTemplateDetails[str] = ApiField(alias='Country')
     custom_field_attributes: CustomFieldAttributes = ApiField(alias='CustomFieldAttributes')
+    disabled: bool = ApiField(alias='Disabled')
     dn: str = ApiField(alias='Dn')
     guid: str = ApiField(alias='Guid')
     id: int = ApiField(alias='Id')
-    ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
-    key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
-    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
+    ip_address_restriction: Items[str] = ApiField(alias='IPAddressRestriction')
+    key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='KeyAlgorithm')
+    key_storage_location: EnvironmentTemplateDetails[str] = ApiField(alias='KeyStorageLocation')
     key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
-    organization: InfoValue[str] = ApiField(alias='Organization')
-    organization_unit: InfoValue[str] = ApiField(alias='OrganizationUnit')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
+    organization: EnvironmentTemplateDetails[str] = ApiField(alias='Organization')
+    organizational_unit: EnvironmentTemplateDetails[str] = ApiField(alias='OrganizationalUnit')
     per_user: bool = ApiField(alias='PerUser')
-    san_email: InfoValue[str] = ApiField(alias='SanEmail')
-    state: InfoValue[str] = ApiField(alias='State')
+    san_email: EnvironmentTemplateDetails[str] = ApiField(alias='SANEmail')
+    state: EnvironmentTemplateDetails[str] = ApiField(alias='State')
     status: int = ApiField(alias='Status')
-    target_store: str = ApiField(alias='TargetStore')
-    template_dn: str = ApiField(alias='TemplateDn')
+    target_store: EnvironmentTemplateDetails[str] = ApiField(alias='TargetStore')
+    template_dn: str = ApiField(alias='TemplateDN')
     type: EnvironmentType = ApiField(alias='Type')
 
 
@@ -180,7 +196,7 @@ class CSPTemplate(ObjectModel):
     expiration: InfoValue[str] = ApiField(alias='Expiration')
     guid: str = ApiField(alias='Guid')
     id: int = ApiField(alias='Id')
-    key_container_dn: str = ApiField(alias='KeyContainerDn')
+    key_container_dn: str = ApiField(alias='KeyContainerDN')
     key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
     max_uses: InfoValue[str] = ApiField(alias='MaxUses')
     object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
@@ -196,19 +212,19 @@ class CSPEnvironment(ObjectModel):
     disabled: bool = ApiField(alias='Disabled')
     dn: str = ApiField(alias='Dn')
     encryption_key_algorithm: EnvironmentTemplateDetails = ApiField(alias='EncryptionKeyAlgorithm')
-    encryption_key_dn: str = ApiField(alias='EncryptionKeyDn')
+    encryption_key_dn: str = ApiField(alias='EncryptionKeyDN')
     expiration: int = ApiField(alias='Expiration')
     guid: str = ApiField(alias='Guid')
     id: int = ApiField(alias='Id')
     ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
     key_storage_location: EnvironmentTemplateDetails = ApiField(alias='KeyStorageLocation')
     key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     max_uses: int = ApiField(alias='MaxUses')
     per_user: bool = ApiField(alias='PerUser')
     signing_key_algorithm: EnvironmentTemplateDetails = ApiField(alias='SigningKeyAlgorithm')
-    signing_key_dn: str = ApiField(alias='SigningKeyDn')
-    template_dn: str = ApiField(alias='TemplateDn')
+    signing_key_dn: str = ApiField(alias='SigningKeyDN')
+    template_dn: str = ApiField(alias='TemplateDN')
 
 
 class DotNetTemplate(ObjectModel):
@@ -220,9 +236,9 @@ class DotNetTemplate(ObjectModel):
     guid: str = ApiField(alias='Guid')
     id: int = ApiField(alias='Id')
     key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
-    key_container_dn: InfoValue[str] = ApiField(alias='KeyContainerDn')
+    key_container_dn: InfoValue[str] = ApiField(alias='KeyContainerDN')
     key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     max_uses: InfoValue[str] = ApiField(alias='MaxUses')
     object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
     per_user: bool = ApiField(alias='PerUser')
@@ -240,13 +256,13 @@ class DotNetEnvironment(ObjectModel):
     id: int = ApiField(alias='Id')
     ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
     key_algorithm: EnvironmentTemplateDetails = ApiField(alias='KeyAlgorithm')
-    key_dn: str = ApiField(alias='KeyDn')
+    key_dn: str = ApiField(alias='KeyDN')
     key_storage_location: EnvironmentTemplateDetails = ApiField(alias='KeyStorageLocation')
     key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     max_uses: int = ApiField(alias='MaxUses')
     per_user: bool = ApiField(alias='PerUser')
-    template_dn: str = ApiField(alias='TemplateDn')
+    template_dn: str = ApiField(alias='TemplateDN')
     type: TemplateType = ApiField(alias='Type')
 
 
@@ -262,7 +278,7 @@ class GPGTemplate(ObjectModel):
     id: int = ApiField(alias='Id')
     key_container_dn: str = ApiField(alias='KeyContainerDN')
     key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     max_uses: InfoValue[str] = ApiField(alias='MaxUses')
     object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
     per_user: bool = ApiField(alias='PerUser')
@@ -288,14 +304,14 @@ class GPGEnvironment(ObjectModel):
     ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
     key_storage_location: EnvironmentTemplateDetails = ApiField(alias='KeyStorageLocation')
     key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     max_uses: int = ApiField(alias='MaxUses')
     per_user: bool = ApiField(alias='PerUser')
     real_name: EnvironmentTemplateDetails = ApiField(alias='RealName')
     read_only: bool = ApiField(alias='ReadOnly')
     signing_key_algorithm: EnvironmentTemplateDetails = ApiField(alias='SigningKeyAlgorithm')
     status: int = ApiField(alias='Status')
-    template_dn: str = ApiField(alias='TemplateDn')
+    template_dn: str = ApiField(alias='TemplateDN')
     type: EnvironmentType = ApiField(alias='Type')
 
 
@@ -308,7 +324,7 @@ class KeyPairTemplate(ObjectModel):
     guid: str = ApiField(alias='Guid')
     id: int = ApiField(alias='Id')
     key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
-    key_container_dn: str = ApiField(alias='KeyContainerDn')
+    key_container_dn: str = ApiField(alias='KeyContainerDN')
     key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
     max_uses: int = ApiField(alias='MaxUses')
     object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
@@ -327,13 +343,13 @@ class KeyPairEnvironment(ObjectModel):
     id: int = ApiField(alias='Id')
     ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
     key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
-    key_dn: str = ApiField(alias='KeyDn')
+    key_dn: str = ApiField(alias='KeyDN')
     key_pair_template: KeyPairTemplate = ApiField(alias='KeyPairTemplate')
     key_storage_location: EnvironmentTemplateDetails = ApiField(alias='KeyStorageLocation')
     key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
-    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDn')
+    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
     status: int = ApiField(alias='Status')
-    template_dn: str = ApiField(alias='TemplateDn')
+    template_dn: str = ApiField(alias='TemplateDN')
     type: EnvironmentType = ApiField(alias='Type')
 
 
@@ -352,7 +368,7 @@ class Application(ObjectModel):
 
 
 class ApplicationCollection(ObjectModel):
-    application_dns: Items[str] = ApiField(alias='ApplicationDns')
+    application_dns: Items[str] = ApiField(alias='ApplicationDNs')
     dn: str = ApiField(alias='Dn')
     guid: str = ApiField(alias='Guid')
     id: int = ApiField(alias='Id')
@@ -361,7 +377,7 @@ class ApplicationCollection(ObjectModel):
 class GlobalConfiguration(ObjectModel):
     approved_key_storage_locations: Items[str] = ApiField(alias='ApprovedKeyStorageLocations')
     available_key_storage_locations: Items[str] = ApiField(alias='AvailableKeyStorageLocations')
-    default_ca_container: str = ApiField(alias='DefaultCaContainer')
+    default_ca_container: str = ApiField(alias='DefaultCAContainer')
     default_certificate_container: str = ApiField(alias='DefaultCertificateContainer')
     default_credential_container: str = ApiField(alias='DefaultCredentialContainer')
     key_use_timeout: int = ApiField(alias='KeyUseTimeout')
@@ -370,11 +386,11 @@ class GlobalConfiguration(ObjectModel):
 
 
 class Project(ObjectModel):
-    application_dns: Items[str] = ApiField(alias='ApplicationDns')
+    application_dns: Items[str] = ApiField(alias='ApplicationDNs')
     applications: List[Application] = ApiField(alias='Applications', default_factory=list)
     auditors: Items[str] = ApiField(alias='Auditors')
     certificate_environments: List[CertificateEnvironment] = ApiField(alias='CertificateEnvironments', default_factory=list)
-    collections: List[SignApplicationCollection] = ApiField(alias='Collections', default_factory=list)
+    collections: List[ApplicationCollection] = ApiField(alias='Collections', default_factory=list)
     created_on: datetime = ApiField(alias='CreatedOn')
     csp_environments: List[CSPEnvironment] = ApiField(alias='CSPEnvironments', default_factory=list)
     custom_field_attributes: Items[CustomFieldAttributes] = ApiField(alias='CustomFieldAttributes', default_factory=list)
@@ -424,3 +440,55 @@ class Rights(ObjectModel):
     @property
     def approve_use(self) -> int:
         return self.value & 64 == 64
+
+class ArchiveFilter(ObjectModel):
+    application_hash: str = ApiField(alias='ApplicationHash')
+    artifact: str = ApiField(alias='Artifact')
+    authenticated_user: str = ApiField(alias='AuthenticatedUser')
+    client_interface: int = ApiField(alias='ClientInterface')
+    client_library_name: str = ApiField(alias='ClientLibraryName')
+    client_library_version: str = ApiField(alias='ClientLibraryVersion')
+    client_mechanism: str = ApiField(alias='ClientMechanism')
+    command: str = ApiField(alias='Command')
+    environment_dn: str = ApiField(alias='EnvironmentDN')
+    event_id: int = ApiField(alias='EventId')
+    executable: str = ApiField(alias='Executable')
+    executable_location: str = ApiField(alias='ExecutableLocation')
+    flow_dn: str = ApiField(alias='FlowDN')
+    grouping: int = ApiField(alias='Grouping')
+    ip_address: str = ApiField(alias='IPAddress')
+    key_type: str = ApiField(alias='KeyType')
+    key_vault_id: int = ApiField(alias='KeyVaultId')
+    machine: str = ApiField(alias='Machine')
+    mechanism: int = ApiField(alias='Mechanism')
+    project_dn: str = ApiField(alias='ProjectDN')
+    project_dns: str = ApiField(alias='ProjectDNs')
+    remote_account: str = ApiField(alias='RemoteAccount')
+    timestamp_after: datetime = ApiField(alias='TimestampAfter')
+    timestamp_before: datetime = ApiField(alias='TimestampBefore')
+
+class ArchiveResults(ObjectModel):
+    application_hash: str = ApiField(alias='ApplicationHash')
+    archive_entry_id: str = ApiField(alias='ArchiveEntryId')
+    artifact: str = ApiField(alias='Artifact')
+    authenticated_user: str = ApiField(alias='AuthenticatedUser')
+    client_library_location: str = ApiField(alias='ClientLibraryLocation')
+    client_library_name: str = ApiField(alias='ClientLibraryName')
+    client_library_version: str = ApiField(alias='ClientLibraryVersion')
+    client_mechanism: str = ApiField(alias='ClientMechanism')
+    command: str = ApiField(alias='Command')
+    environment: str = ApiField(alias='Environment')
+    event_id: int = ApiField(alias='EventId')
+    executable: str = ApiField(alias='Executable')
+    executable_location: str = ApiField(alias='ExecutableLocation')
+    flow: str = ApiField(alias='Flow')
+    grouping: int = ApiField(alias='Grouping')
+    ip_address: str = ApiField(alias='IPAddress')
+    key_type: str = ApiField(alias='KeyType')
+    key_vault_id: int = ApiField(alias='KeyVaultId')
+    machine: str = ApiField(alias='Machine')
+    mechanism: int = ApiField(alias='Mechanism')
+    platform: str = ApiField(alias='Platform')
+    project: str = ApiField(alias='Project')
+    remote_account: str = ApiField(alias='RemoteAccount')
+    timestamp: datetime = ApiField(alias='Timestamp')
