@@ -199,6 +199,11 @@ class Classes(metaclass=IterableMeta):
     intermediate_and_root_certificates = "Intermediate and Root Certificates"
     jks = "JKS"
     jks_trust_store = "JKS Trust Store"
+    jss_cluster = "JSS Cluster"
+    jss_container = "JSS Container"
+    jss_discovery = "JSS Discovery"
+    jss_discovery_manager = "JSS Discovery Manager"
+    jss_namespace = "JSS Namespace"
     jump_server = "Jump Server"
     juniper_sas = "Juniper SAS"
     key_base = "Key Base"
